; --------------------------------------------------------------------------------
; I2C SCANNER                                                           25/03/2020
;
; STATUS: FUNCIONA
;
; Varre o bus, controlado pelos pinos RA1 (SDA) e RA2 (SCL), a 
; procura de dispositivos I2C conectados e, ao encontrar um endereco
; valido, o programa mostra o resultado em 8 LEDs conectados ao PORTB
;
; Serao pesquisados os provaveis enderecos de escrita de dispositivos 
; (enderecos pares de 0 a 254) que podem estar conectados no bus I2C. 
; A pesquisa se dara do endereco mais alto (254) para o mais baixo (2).
;
; O mecanismo de pesquisa consiste em pegar um numero de endereco par, que
; caracteriza um endereco de escrita, e, apos enviar uma condicao de START
; ao bus I2C, esse endereco sera enviado pelo bus esperando-se que haja
; resposta positiva (ACK) de algum dispositivo conectado ao bus.
;
; Ao ser ligado, o circuito acentdera os LEDs vermelho e verde conectados 
; aos pinos RA0 e RA3, respectivamente, e ficara aguardando que o botao
; conectado ao pino Ra7 seja pressionado para iniciar a pesquisa.
;
; Ao iniciar a pesquisa o LED vermelho conectado ao pino RA0 piscara ate
; que um dispositivo I2C seja encontrado no bus.
;
; O recebimento de uma resposta positiva (ACK), significa que um dispositivo
; I2C foi encontrado. O LED verde no pino RA3 acendera e o LED vermelho do pino 
; RA0 apagara, indicando que o valor mostrado nos 8 LEDs do PORTB e' um 
; endereco valido. O programa fica aguardando ate que o botao conectado no 
; pino RA7 seja pressionado para continuar a pesquisa e o processo se repetira
; ate que um outro dispositivo I2C seja encontrado ou ate que se esgote os 
; enderecos possiveis.
;
; O fim da pesquisa sera indicado com os LEDs verde (RA3) e vermelho (RA0) acesos.
;
; --------------------------------------------------------------------------------

; --------------------------------------------------------------------------------
; NOTAS:
; 1 - Os pinos RA1 e RA2 do bus I2C devem ser conectados ao +5V com resistores 
;     entre 4K e 10k.
; 2 - Se estiver usando programacao ICSP, os pinos RB6/PGC e RB7/PGD devem
;     ter os LEDs conectados a eles por resistores de no minimo 1K.
; 3 - Teoricamente, qualquer dispositivi I2C pode ser usado como:
;     LCDs, RTC DS1307, expansor de I/I PCF8574, etc.
; --------------------------------------------------------------------------------

; --------------------------------------------------------------------------------
; HARDWARE
;   Fosc ............... 4MHZ interno
;   RA1 ................ Bus I2C --> SDA - com resistor pull-up (4k a 10K)
;   RA2 ................ Bus I2C --> SCL - com resistor pull-up (4k a 10K)
;   RA0 ................ LED Vermelho
;   RA3 ................ LDE Verde
;   RA7 ................ Botao com resistor pull-up 10K (active low)
;   RB0 a RB7 .......... LEDs
;
; --------------------------------------------------------------------------------
	list      p=16f628a
    #include <p16F628A.inc>
    errorlevel  -302
;
	__CONFIG   _CP_OFF &  _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT
;  __CONFIG   _CP_OFF &  _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _HS_OSC

    #include  "familias.inc"
;
              UDATA   0x20 
;
CLOCK         equ     .4000000
WRK1          res     .1
addr          res     .1            
delay_temp    res     .1                ; para o modulo delays
; --------------------------------------------------------------------------
; Registradores e simbolos do servico 'i2cproto_services.inc'
; --------------------------------------------------------------------------
PULSE_DUTY     equ       .25            ; duty cycle do pulso de clock em microsegundos
                                        ; 25us e' ideal para Fosc ate' 20MHZ
i2c_N0         RES       1              ; sub-rotinas I2C
i2c_IBUFF      RES       1              ; sub-rotinas I2C - buffer de leitura
i2c_OBUFF      RES       1              ; sub-rotinas I2C - buffer de gravacao
#define     i2c_TRIS     TRISA          ; <--- MODIFIQUE - registrado de controle de I/O da porta I2C    
#define     i2c_PORT     PORTA          ; <--- MODIFIQUE - porta de I/O do I2C
#define     i2c_SDA      1              ; <--- MODIFIQUE - bit SDA do I2C
#define     i2c_SCL      2              ; <--- MODIFIQUE - bit SCL do I2C
; --------------------------------------------------------------------------
; Botao 
; --------------------------------------------------------------------------
#define     boTris       TRISA
#define     boPort       PORTA
#define     boBit        .7
; --------------------------------------------------------------------------
; LED VERDE 
; --------------------------------------------------------------------------
#define     LEDVDTRIS    TRISA
#define     LEDVDPORT    PORTA
#define     LEDVD        .3
; --------------------------------------------------------------------------
; LED VERMELHO
; --------------------------------------------------------------------------
#define     LEDVMTRIS    TRISA
#define     LEDVMPORT    PORTA
#define     LEDVM        .0

;
; --------------------------------------------------------------------------
;
RESET_VECTOR CODE  0x0000
           goto    inicio
INT_VECTOR CODE    0x0004
	       retfie
;
MAIN       CODE
           #include "i2cproto_services.inc"
; ---------------------------------------------------------------------------------------
;            CONFIGURACOES INICIAIS
; ---------------------------------------------------------------------------------------
inicio     movlw   0x07
           movwf   CMCON
           clrf    PORTA
           clrf    PORTB
           bsf     STATUS,RP0
           movwf   0x80                  ; RA7 --> entrada do botao 
           movwf   TRISA                 ; 
           clrf    TRISB                 ; saida para 8 LEDs     
           bcf     LEDVDTRIS, LEDVD
           bcf     LEDVMTRIS, LEDVM
           bcf     STATUS,RP0

; -----------------------------------------------------------------------------
; Rotina principal
; -----------------------------------------------------------------------------
           bsf     LEDVMPORT, LEDVM      ; acende LED vermelho
           bsf     LEDVDPORT, LEDVD      ; acende LED verde
           movlw   .0                    ; W=0
           movwf   PORTB                 ; apaga todos os LEDs do PCF8574
           call    waitOper              ; espera o operador iniciar a pesquisa
scan       movlw   .254                  ; endereco de write mais alto possivel
           movwf   addr                  ; carrega contador
pickadd    bsf     LEDVMPORT, LEDVM      ; acende o LED vermelho
           bcf     LEDVDPORT, LEDVD      ; apaga o LED verde
           call    i2cStop               ; certifica que esta em STOP
           nop                           ; tempo
           nop                           ; tempo
           call    i2cStart              ; envia START
           movf    addr, w               ; provavel endereco de write
           call    i2cPut                ; envia endereco
           call    i2cGetACK             ; espera ACK/NACK 
           btfss   i2c_IBUFF, i2c_SDA    ; recebeu NACK???
           goto    gotAck                ; NAO --> e' ACK
           bcf     LEDVMPORT, LEDVM      ; apaga o LED vermelho
           call    i2cStop               ; SIM --> endereco invalido
;
           movf    addr, w               ; pega o valor
           movwf   PORTB                 ; envia para os LEDs 
           movlw   .20                   ; 20ms
           call    delay_ms              ; tempo de visualizacao dos LDEs
nextadd    decf    addr, f               ; exclui provavel endereco de escrita
           decfsz  addr, f               ; proximo endereco
           goto    pickadd               ; vai testar
           bsf     LEDVMPORT, LEDVM      ; acende o LED vermelho
           bsf     LEDVDPORT, LEDVD      ; acende o LED verde
           movlw   .0                    ; W = 0
           movwf   PORTB                 ; apaga os 8 LEDs do PCF8474
           call    waitOper              ; espera vontade do operador
fim        goto    scan                  ; novamente...
           
; -----------------------------------------------------------------------------
; Transforma endereco de 8 bits (R/W) no endereco do dispositivo I2C de 7 bits
; e mostra no LCD.
; -----------------------------------------------------------------------------
gotAck     bcf     LEDVMPORT, LEDVM      ; apaga o LED vermelho
           bsf     LEDVDPORT, LEDVD      ; acende o LED verde
           movf    addr, w               ; endereco de write que respondeu ACK
           movwf   WRK1                    ; carrega WRK1
           rrf     WRK1, f                 ; converte para endereco de 7 bits
           movf    WRK1, w                 ; pega o device address
           andlw   0x7F                  ; desliga o bit mais alto
           movwf   PORTB                 ; mosta nos LEDs
           call    waitOper              ; espera vontade do operador
           goto    nextadd               ; proximo endereco

; -----------------------------------------------------------------------------
; Espera o usuario pressionar o botao antes de prosseguir.
; ----------------------------------------------------------------------------- 
waitOper   bsf     STATUS, RP0           ; banco 1
           bsf     boTris, boBit         ; entrada do botao
           bcf     STATUS, RP0           ; banco 0
boGet      movf    boPort, w             ; le a porta do botao
           movwf   WRK1                    ; salva a leitura
           movlw   .60                   ; 60ms 
           call    delay_ms              ; debounce do botao
           btfsc   WRK1, boBit             ; botao pressionado?
           goto    boGet                 ; NAO --> espera alguem pressionar
boFree     movf    boPort, w             ; le a porta do botao
           movwf   WRK1                    ; salva a leitura
           movlw   .60                   ; 60ms
           call    delay_ms              ; debounce do botao
           btfss   WRK1, boBit             ; o botao foi liberado?
           goto    boFree                ; NAO --> espera liberar
           bcf     LEDVMPORT, LEDVM      ; apaga LED vermelho
           bcf     LEDVDPORT, LEDVD      ; apaga LED verde
           return                        ; fim


           END
